Class: HaveAPI::ModelAdapters::ActiveRecord::Output
- Inherits:
-
HaveAPI::ModelAdapter::Output
- Object
- HaveAPI::ModelAdapter::Output
- HaveAPI::ModelAdapters::ActiveRecord::Output
- Defined in:
- lib/haveapi/model_adapters/active_record.rb
Class Method Summary collapse
Instance Method Summary collapse
Methods inherited from HaveAPI::ModelAdapter::Output
Constructor Details
This class inherits a constructor from HaveAPI::ModelAdapter::Output
Class Method Details
.used_by(action) ⇒ Object
225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 |
# File 'lib/haveapi/model_adapters/active_record.rb', line 225 def self.used_by(action) action.(:object) do output do custom :path_params, label: 'URL parameters', desc: 'An array of parameters needed to resolve URL to this object' bool :resolved, label: 'Resolved', desc: 'True if the association is resolved' end end return unless i[object object_list].include?(action.input.layout) clean = proc do |raw| if raw.is_a?(String) raw.strip.split(',') elsif raw.is_a?(Array) raw end end desc = " A list of names of associated resources separated by a comma.\n Nested associations are declared with '__' between resource names.\n For example, 'user,node' will resolve the two associations.\n To resolve further associations of node, use e.g. 'user,node__location',\n to go even deeper, use e.g. 'user,node__location__environment'.\n END\n\n action.meta(:global) do\n input do\n custom :includes, label: 'Included associations',\n desc:, &clean\n end\n end\n\n action.send(:include, Action::InstanceMethods)\nend\n" |
Instance Method Details
#[](name) ⇒ Object
267 268 269 270 271 272 273 274 275 276 |
# File 'lib/haveapi/model_adapters/active_record.rb', line 267 def [](name) param = @context.action.output[name] v = @object.send(param.db_name) if v.is_a?(::ActiveRecord::Base) resourcify(param, v) else v end end |
#has_param?(name) ⇒ Boolean
262 263 264 265 |
# File 'lib/haveapi/model_adapters/active_record.rb', line 262 def has_param?(name) param = @context.action.output[name] param && @object.respond_to?(param.db_name) end |
#meta ⇒ Object
278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 |
# File 'lib/haveapi/model_adapters/active_record.rb', line 278 def res = @context.action.resource params = if @context.action.name.demodulize == 'Index' \ && !@context.action.resolve \ && res.const_defined?(:Show) res::Show.resolve_path_params(@object) else @context.action.resolve_path_params(@object) end { path_params: params.is_a?(Array) ? params : [params], resolved: true } end |