Class: HaveAPI::Action

Inherits:
Common
  • Object
show all
Includes:
Hookable
Defined in:
lib/haveapi/action.rb

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Hookable

included

Methods inherited from Common

check_build, has_attr, inherit_attrs

Constructor Details

#initialize(request, version, params, body, context) ⇒ Action

Returns a new instance of Action.



291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
# File 'lib/haveapi/action.rb', line 291

def initialize(request, version, params, body, context)
  @request = request
  @version = version
  @params = params
  @params.update(body) if body
  @context = context
  @context.action = self.class
  @context.action_instance = self
  @metadata = {}
  @reply_meta = {object: {}, global: {}}
  @flags = {}

  class_auth = self.class.authorization

  if class_auth
    @authorization = class_auth.clone
  else
    @authorization = Authorization.new {}
  end

  ret = call_class_hooks_as_for(
    Action,
    :pre_authorize,
    args: [@context],
    initial: {blocks: []},
  )

  ret[:blocks].reverse_each do |block|
    @authorization.prepend_block(block)
  end
end

Class Attribute Details

.authorizationObject (readonly)

Returns the value of attribute authorization.



47
48
49
# File 'lib/haveapi/action.rb', line 47

def authorization
  @authorization
end

.examplesObject (readonly)

Returns the value of attribute examples.



47
48
49
# File 'lib/haveapi/action.rb', line 47

def examples
  @examples
end

.resourceObject

Returns the value of attribute resource.



46
47
48
# File 'lib/haveapi/action.rb', line 46

def resource
  @resource
end

Instance Attribute Details

#errorsObject (readonly)

Returns the value of attribute errors.



42
43
44
# File 'lib/haveapi/action.rb', line 42

def errors
  @errors
end

#flagsObject

Returns the value of attribute flags.



43
44
45
# File 'lib/haveapi/action.rb', line 43

def flags
  @flags
end

#messageObject (readonly)

Returns the value of attribute message.



42
43
44
# File 'lib/haveapi/action.rb', line 42

def message
  @message
end

#versionObject (readonly)

Returns the value of attribute version.



42
43
44
# File 'lib/haveapi/action.rb', line 42

def version
  @version
end

Class Method Details

.action_nameObject



194
195
196
# File 'lib/haveapi/action.rb', line 194

def action_name
  (@action_name ? @action_name.to_s : to_s).demodulize
end

.action_name=(name) ⇒ Object



198
199
200
# File 'lib/haveapi/action.rb', line 198

def action_name=(name)
  @action_name = name
end

.authorize(&block) ⇒ Object



183
184
185
# File 'lib/haveapi/action.rb', line 183

def authorize(&block)
  @authorization = Authorization.new(&block)
end

.build_route(prefix) ⇒ Object



202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
# File 'lib/haveapi/action.rb', line 202

def build_route(prefix)
  route = @route || action_name.underscore
    if @route
      @route
    elsif action_name
      action_name.to_s.demodulize.underscore
    else
      to_s.demodulize.underscore
    end

  if !route.is_a?(String) && route.respond_to?(:call)
    route = route.call(self.resource)
  end

  prefix + route % {resource: self.resource.resource_name.underscore}
end

.delayed_inherited(subclass) ⇒ Object



60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
# File 'lib/haveapi/action.rb', line 60

def delayed_inherited(subclass)
  resource = subclass.resource || Kernel.const_get(subclass.to_s.deconstantize)

  inherit_attrs(subclass)
  inherit_attrs_from_resource(subclass, resource, [:auth])

  i = @input.clone
  i.action = subclass

  o = @output.clone
  o.action = subclass

  m = {}

  @meta.each do |k,v|
    m[k] = v && v.clone
    next unless v
    m[k].action = subclass
  end

  subclass.instance_variable_set(:@input, i)
  subclass.instance_variable_set(:@output, o)
  subclass.instance_variable_set(:@meta, m)

  begin
    subclass.instance_variable_set(:@resource, resource)
    subclass.instance_variable_set(:@model, resource.model)
    resource.action_defined(subclass)
  rescue NoMethodError
    return
  end
end

.describe(context) ⇒ Object



219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
# File 'lib/haveapi/action.rb', line 219

def describe(context)
  authorization = (@authorization && @authorization.clone) || Authorization.new

  if (context.endpoint || context.current_user) \
      && !authorization.authorized?(context.current_user, context.path_params_from_args)
    return false
  end

  route_method = context.action.http_method.to_s.upcase
  context.authorization = authorization

  if context.endpoint
    context.action_instance = context.action.from_context(context)

    ret = catch(:return) do
      context.action_prepare = context.action_instance.prepare
    end

    return false if ret == false
  end

  {
    auth: @auth,
    description: @desc,
    aliases: @aliases,
    blocking: @blocking ? true : false,
    input: @input ? @input.describe(context) : {parameters: {}},
    output: @output ? @output.describe(context) : {parameters: {}},
    meta: @meta ? @meta.merge(@meta) { |_, v| v && v.describe(context) } : nil,
    examples: @examples ? @examples.describe(context) : [],
    scope: context.action_scope,
    path: context.resolved_path,
    method: route_method,
    help: "#{context.path}?method=#{route_method}",
  }
end

.example(title = '', &block) ⇒ Object



187
188
189
190
191
192
# File 'lib/haveapi/action.rb', line 187

def example(title = '', &block)
  @examples ||= ExampleList.new
  e = Example.new(title)
  e.instance_eval(&block)
  @examples << e
end

.from_context(c) ⇒ Object



270
271
272
273
274
275
276
277
278
279
# File 'lib/haveapi/action.rb', line 270

def from_context(c)
  ret = new(nil, c.version, c.params, nil, c)
  ret.instance_exec do
    @safe_params = @params.dup
    @authorization = c.authorization
    @current_user = c.current_user
  end

  ret
end

.inherit_attrs_from_resource(action, r, attrs) ⇒ Object

Inherit attributes from resource action is defined in.



257
258
259
260
261
262
263
264
265
266
267
268
# File 'lib/haveapi/action.rb', line 257

def inherit_attrs_from_resource(action, r, attrs)
  begin
    return unless r.obj_type == :resource

  rescue NoMethodError
    return
  end

  attrs.each do |attr|
    action.method(attr).call(r.method(attr).call)
  end
end

.inherited(subclass) ⇒ Object



49
50
51
52
53
54
55
56
57
58
# File 'lib/haveapi/action.rb', line 49

def inherited(subclass)
  # puts "Action.inherited called #{subclass} from #{to_s}"

  subclass.instance_variable_set(:@obj_type, obj_type)

  if subclass.name
    # not an anonymouse class
    delayed_inherited(subclass)
  end
end

.initializeObject



93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
# File 'lib/haveapi/action.rb', line 93

def initialize
  return if @initialized

  check_build("#{self}.input") do
    input.exec
    model_adapter(input.layout).load_validators(model, input) if model
  end

  check_build("#{self}.output") do
    output.exec
  end

  model_adapter(input.layout).used_by(:input, self)
  model_adapter(output.layout).used_by(:output, self)

  if blocking
    meta(:global) do
      output do
        integer :action_state_id,
            label: 'Action state ID',
            desc: 'ID of ActionState object for state querying. When null, the action '+
                  'is not blocking for the current invocation.'
      end
    end
  end

  if @meta
    @meta.each_value do |m|
      next unless m

      check_build("#{self}.meta.input") do
        m.input && m.input.exec
      end

      check_build("#{self}.meta.output") do
        m.output && m.output.exec
      end
    end
  end

  @initialized = true
end

.input(layout = nil, namespace: nil, &block) ⇒ Object



150
151
152
153
154
155
156
157
158
159
# File 'lib/haveapi/action.rb', line 150

def input(layout = nil, namespace: nil, &block)
  if block
    @input ||= Params.new(:input, self)
    @input.layout = layout
    @input.namespace = namespace
    @input.add_block(block)
  else
    @input
  end
end

.meta(type = :object, &block) ⇒ Object



172
173
174
175
176
177
178
179
180
181
# File 'lib/haveapi/action.rb', line 172

def meta(type = :object, &block)
  if block
    @meta ||= {object: nil, global: nil}
    @meta[type] ||= Metadata::ActionMetadata.new
    @meta[type].action = self
    @meta[type].instance_exec(&block)
  else
    @meta[type]
  end
end

.model_adapter(layout) ⇒ Object



146
147
148
# File 'lib/haveapi/action.rb', line 146

def model_adapter(layout)
  ModelAdapter.for(layout, resource.model)
end

.output(layout = nil, namespace: nil, &block) ⇒ Object



161
162
163
164
165
166
167
168
169
170
# File 'lib/haveapi/action.rb', line 161

def output(layout = nil, namespace: nil, &block)
  if block
    @output ||= Params.new(:output, self)
    @output.layout = layout
    @output.namespace = namespace
    @output.add_block(block)
  else
    @output
  end
end

.resolve_path_params(object) ⇒ Object



281
282
283
284
285
286
287
288
# File 'lib/haveapi/action.rb', line 281

def resolve_path_params(object)
  if self.resolve
    self.resolve.call(object)

  else
    object.respond_to?(:id) ? object.id : nil
  end
end

.validate_buildObject



136
137
138
139
140
141
142
143
144
# File 'lib/haveapi/action.rb', line 136

def validate_build
  check_build("#{self}.input") do
    input.validate_build
  end

  check_build("#{self}.output") do
    output.validate_build
  end
end

Instance Method Details

#authorized?(user) ⇒ Boolean

Returns:



331
332
333
334
# File 'lib/haveapi/action.rb', line 331

def authorized?(user)
  @current_user = user
  @authorization.authorized?(user, extract_path_params)
end

#current_userObject



336
337
338
# File 'lib/haveapi/action.rb', line 336

def current_user
  @current_user
end

#execObject

This method must be reimplemented in every action. It must not be invoked directly, only via safe_exec, which restricts output.



377
378
379
# File 'lib/haveapi/action.rb', line 377

def exec
  ['not implemented']
end

#inputObject



344
345
346
# File 'lib/haveapi/action.rb', line 344

def input
  @safe_params[ self.class.input.namespace ] if self.class.input
end

#metaObject



352
353
354
# File 'lib/haveapi/action.rb', line 352

def meta
  @metadata
end

#paramsObject



340
341
342
# File 'lib/haveapi/action.rb', line 340

def params
  @safe_params
end

#pre_execObject



371
372
373
# File 'lib/haveapi/action.rb', line 371

def pre_exec

end

#prepareObject

FIXME: is this correct behaviour? ++



367
368
369
# File 'lib/haveapi/action.rb', line 367

def prepare

end

#requestObject



348
349
350
# File 'lib/haveapi/action.rb', line 348

def request
  @request
end

#safe_execObject

Calls exec while catching all exceptions and restricting output only to what user can see. Return array [status, data|error, errors]



384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
# File 'lib/haveapi/action.rb', line 384

def safe_exec
  exec_ret = catch(:return) do
    begin
      validate!
      prepare
      pre_exec
      exec
    rescue Exception => e
      tmp = call_class_hooks_as_for(Action, :exec_exception, args: [@context, e])

      if tmp.empty?
        p e.message
        puts e.backtrace
        error('Server error occurred')
      end

      unless tmp[:status]
        error(tmp[:message], {}, http_status: tmp[:http_status] || 500)
      end
    end
  end

  begin
    output_ret = safe_output(exec_ret)
  rescue Exception => e
    tmp = call_class_hooks_as_for(Action, :exec_exception, args: [@context, e])

    p e.message
    puts e.backtrace

    return [
      tmp[:status] || false,
      tmp[:message] || 'Server error occurred',
      {},
      tmp[:http_status] || 500,
    ]
  end

  output_ret
end

#safe_output(ret) ⇒ Object



429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
# File 'lib/haveapi/action.rb', line 429

def safe_output(ret)
  if ret
    output = self.class.output

    if output
      safe_ret = nil
      adapter = self.class.model_adapter(output.layout)
      out_params = self.class.output.params

      case output.layout
        when :object
          out = adapter.output(@context, ret)
          safe_ret = @authorization.filter_output(
            out_params,
            out,
            true
          )
          @reply_meta[:global].update(out.meta)

        when :object_list
          safe_ret = []

          ret.each do |obj|
            out = adapter.output(@context, obj)

            safe_ret << @authorization.filter_output(
              out_params,
              out,
              true
            )
            safe_ret.last.update({Metadata.namespace => out.meta}) unless meta[:no]
          end

        when :hash
          safe_ret = @authorization.filter_output(
            out_params,
            adapter.output(@context, ret),
            true
          )

        when :hash_list
          safe_ret = ret
          safe_ret.map! do |hash|
            @authorization.filter_output(
              out_params,
              adapter.output(@context, hash),
              true
            )
          end

        else
          safe_ret = ret
      end

      if self.class.blocking
        @reply_meta[:global][:action_state_id] = state_id
      end

      ns = {output.namespace => safe_ret}
      ns[Metadata.namespace] = @reply_meta[:global] unless meta[:no]

      [true, ns]

    else
      [true, {}]
    end

  else
    [false, @message, @errors, @http_status]
  end
end

#set_meta(hash) ⇒ Object



356
357
358
# File 'lib/haveapi/action.rb', line 356

def set_meta(hash)
  @reply_meta[:global].update(hash)
end

#v?(v) ⇒ Boolean

Returns:



425
426
427
# File 'lib/haveapi/action.rb', line 425

def v?(v)
  @version == v
end

#validate!Object



323
324
325
326
327
328
329
# File 'lib/haveapi/action.rb', line 323

def validate!
  begin
    @params = validate
  rescue ValidationError => e
    error(e.message, e.to_hash)
  end
end