Class: HaveAPI::Action

Inherits:
Common
  • Object
show all
Includes:
Hookable
Defined in:
lib/haveapi/action.rb

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Hookable

included

Methods inherited from Common

check_build, has_attr, inherit_attrs

Constructor Details

#initialize(request, version, params, body, context) ⇒ Action

Returns a new instance of Action.



258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
# File 'lib/haveapi/action.rb', line 258

def initialize(request, version, params, body, context)
  @request = request
  @version = version
  @params = params
  @params.update(body) if body
  @context = context
  @context.action = self.class
  @context.action_instance = self
  @metadata = {}
  @reply_meta = {object: {}, global: {}}
  @flags = {}

  class_auth = self.class.authorization

  if class_auth
    @authorization = class_auth.clone
  else
    @authorization = Authorization.new {}
  end
end

Class Attribute Details

.authorization ⇒ Object (readonly)

Returns the value of attribute authorization.



34
35
36
# File 'lib/haveapi/action.rb', line 34

def authorization
  @authorization
end

.examples ⇒ Object (readonly)

Returns the value of attribute examples.



34
35
36
# File 'lib/haveapi/action.rb', line 34

def examples
  @examples
end

.resource ⇒ Object (readonly)

Returns the value of attribute resource.



34
35
36
# File 'lib/haveapi/action.rb', line 34

def resource
  @resource
end

Instance Attribute Details

#errors ⇒ Object (readonly)

Returns the value of attribute errors.



30
31
32
# File 'lib/haveapi/action.rb', line 30

def errors
  @errors
end

#flags ⇒ Object

Returns the value of attribute flags.



31
32
33
# File 'lib/haveapi/action.rb', line 31

def flags
  @flags
end

#message ⇒ Object (readonly)

Returns the value of attribute message.



30
31
32
# File 'lib/haveapi/action.rb', line 30

def message
  @message
end

#version ⇒ Object (readonly)

Returns the value of attribute version.



30
31
32
# File 'lib/haveapi/action.rb', line 30

def version
  @version
end

Class Method Details

.authorize(&block) ⇒ Object



169
170
171
# File 'lib/haveapi/action.rb', line 169

def authorize(&block)
  @authorization = Authorization.new(&block)
end

.build_route(prefix) ⇒ Object



180
181
182
183
184
185
186
187
188
# File 'lib/haveapi/action.rb', line 180

def build_route(prefix)
  route = @route || to_s.demodulize.underscore

  if !route.is_a?(String) && route.respond_to?(:call)
    route = route.call(self.resource)
  end

  prefix + route % {resource: self.resource.to_s.demodulize.underscore}
end

.delayed_inherited(subclass) ⇒ Object



47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
# File 'lib/haveapi/action.rb', line 47

def delayed_inherited(subclass)
  resource = Kernel.const_get(subclass.to_s.deconstantize)

  inherit_attrs(subclass)
  inherit_attrs_from_resource(subclass, resource, [:auth])

  i = @input.clone
  i.action = subclass

  o = @output.clone
  o.action = subclass

  m = {}

  @meta.each do |k,v|
    m[k] = v && v.clone
    next unless v
    m[k].action = subclass
  end

  subclass.instance_variable_set(:@input, i)
  subclass.instance_variable_set(:@output, o)
  subclass.instance_variable_set(:@meta, m)

  begin
    subclass.instance_variable_set(:@resource, resource)
    subclass.instance_variable_set(:@model, resource.model)
  rescue NoMethodError
    return
  end
end

.describe(context) ⇒ Object



190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
# File 'lib/haveapi/action.rb', line 190

def describe(context)
  authorization = (@authorization && @authorization.clone) || Authorization.new

  return false if (context.endpoint || context.current_user) && !authorization.authorized?(context.current_user)

  route_method = context.action.http_method.to_s.upcase
  context.authorization = authorization

  if context.endpoint
    context.action_instance = context.action.from_context(context)

    ret = catch(:return) do
      context.action_prepare = context.action_instance.prepare
    end

    return false if ret == false
  end

  {
      auth: @auth,
      description: @desc,
      aliases: @aliases,
      blocking: @blocking ? true : false,
      input: @input ? @input.describe(context) : {parameters: {}},
      output: @output ? @output.describe(context) : {parameters: {}},
      meta: @meta ? @meta.merge(@meta) { |_, v| v && v.describe(context) } : nil,
      examples: @examples ? @examples.describe(context) : [],
      url: context.resolved_url,
      method: route_method,
      help: "#{context.url}?method=#{route_method}"
  }
end

.example(title = '', &block) ⇒ Object



173
174
175
176
177
178
# File 'lib/haveapi/action.rb', line 173

def example(title = '', &block)
  @examples ||= ExampleList.new
  e = Example.new(title)
  e.instance_eval(&block)
  @examples << e
end

.from_context(c) ⇒ Object



237
238
239
240
241
242
243
244
245
246
# File 'lib/haveapi/action.rb', line 237

def from_context(c)
  ret = new(nil, c.version, c.params, nil, c)
  ret.instance_exec do
    @safe_params = @params.dup
    @authorization = c.authorization
    @current_user = c.current_user
  end

  ret
end

.inherit_attrs_from_resource(action, r, attrs) ⇒ Object

Inherit attributes from resource action is defined in.



224
225
226
227
228
229
230
231
232
233
234
235
# File 'lib/haveapi/action.rb', line 224

def inherit_attrs_from_resource(action, r, attrs)
  begin
    return unless r.obj_type == :resource

  rescue NoMethodError
    return
  end

  attrs.each do |attr|
    action.method(attr).call(r.method(attr).call)
  end
end

.inherited(subclass) ⇒ Object



36
37
38
39
40
41
42
43
44
45
# File 'lib/haveapi/action.rb', line 36

def inherited(subclass)
  # puts "Action.inherited called #{subclass} from #{to_s}"

  subclass.instance_variable_set(:@obj_type, obj_type)

  if subclass.name
    # not an anonymouse class
    delayed_inherited(subclass)
  end
end

.initialize ⇒ Object



79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
# File 'lib/haveapi/action.rb', line 79

def initialize
  return if @initialized

  check_build("#{self}.input") do
    input.exec
    model_adapter(input.layout).load_validators(model, input) if model
  end

  check_build("#{self}.output") do
    output.exec
  end

  model_adapter(input.layout).used_by(:input, self)
  model_adapter(output.layout).used_by(:output, self)

  if blocking
    meta(:global) do
      output do
        integer :action_state_id,
            label: 'Action state ID',
            desc: 'ID of ActionState object for state querying. When null, the action '+
                  'is not blocking for the current invocation.'
      end
    end
  end

  if @meta
    @meta.each_value do |m|
      next unless m

      check_build("#{self}.meta.input") do
        m.input && m.input.exec
      end

      check_build("#{self}.meta.output") do
        m.output && m.output.exec
      end
    end
  end

  @initialized = true
end

.input(layout = nil, namespace: nil, &block) ⇒ Object



136
137
138
139
140
141
142
143
144
145
# File 'lib/haveapi/action.rb', line 136

def input(layout = nil, namespace: nil, &block)
  if block
    @input ||= Params.new(:input, self)
    @input.layout = layout
    @input.namespace = namespace
    @input.add_block(block)
  else
    @input
  end
end

.meta(type = :object, &block) ⇒ Object



158
159
160
161
162
163
164
165
166
167
# File 'lib/haveapi/action.rb', line 158

def meta(type = :object, &block)
  if block
    @meta ||= {object: nil, global: nil}
    @meta[type] ||= Metadata::ActionMetadata.new
    @meta[type].action = self
    @meta[type].instance_exec(&block)
  else
    @meta[type]
  end
end

.model_adapter(layout) ⇒ Object



132
133
134
# File 'lib/haveapi/action.rb', line 132

def model_adapter(layout)
  ModelAdapter.for(layout, resource.model)
end

.output(layout = nil, namespace: nil, &block) ⇒ Object



147
148
149
150
151
152
153
154
155
156
# File 'lib/haveapi/action.rb', line 147

def output(layout = nil, namespace: nil, &block)
  if block
    @output ||= Params.new(:output, self)
    @output.layout = layout
    @output.namespace = namespace
    @output.add_block(block)
  else
    @output
  end
end

.resolve_url_params(object) ⇒ Object



248
249
250
251
252
253
254
255
# File 'lib/haveapi/action.rb', line 248

def resolve_url_params(object)
  if self.resolve
    self.resolve.call(object)

  else
    object.respond_to?(:id) ? object.id : nil
  end
end

.validate_build ⇒ Object



122
123
124
125
126
127
128
129
130
# File 'lib/haveapi/action.rb', line 122

def validate_build
  check_build("#{self}.input") do
    input.validate_build
  end

  check_build("#{self}.output") do
    output.validate_build
  end
end

Instance Method Details

#authorized?(user) ⇒ Boolean

Returns:



287
288
289
290
# File 'lib/haveapi/action.rb', line 287

def authorized?(user)
  @current_user = user
  @authorization.authorized?(user)
end

#current_user ⇒ Object



292
293
294
# File 'lib/haveapi/action.rb', line 292

def current_user
  @current_user
end

#exec ⇒ Object

This method must be reimplemented in every action. It must not be invoked directly, only via safe_exec, which restricts output.



333
334
335
# File 'lib/haveapi/action.rb', line 333

def exec
  ['not implemented']
end

#input ⇒ Object



300
301
302
# File 'lib/haveapi/action.rb', line 300

def input
  @safe_params[ self.class.input.namespace ] if self.class.input
end

#meta ⇒ Object



308
309
310
# File 'lib/haveapi/action.rb', line 308

def meta
  @metadata
end

#params ⇒ Object



296
297
298
# File 'lib/haveapi/action.rb', line 296

def params
  @safe_params
end

#pre_exec ⇒ Object



327
328
329
# File 'lib/haveapi/action.rb', line 327

def pre_exec

end

#prepare ⇒ Object

FIXME: is this correct behaviour? ++



323
324
325
# File 'lib/haveapi/action.rb', line 323

def prepare

end

#request ⇒ Object



304
305
306
# File 'lib/haveapi/action.rb', line 304

def request
  @request
end

#safe_exec ⇒ Object

Calls exec while catching all exceptions and restricting output only to what user can see. Return array [status, data|error, errors]



340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
# File 'lib/haveapi/action.rb', line 340

def safe_exec
  ret = catch(:return) do
    begin
      validate!
      prepare
      pre_exec
      exec
    rescue Exception => e
      tmp = call_class_hooks_as_for(Action, :exec_exception, args: [@context, e])

      if tmp.empty?
        p e.message
        puts e.backtrace
        error('Server error occurred')
      end

      unless tmp[:status]
        error(tmp[:message], {}, http_status: tmp[:http_status] || 500)
      end
    end
  end

  safe_output(ret)
end

#safe_output(ret) ⇒ Object



369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
# File 'lib/haveapi/action.rb', line 369

def safe_output(ret)
  if ret
    output = self.class.output

    if output
      safe_ret = nil
      adapter = self.class.model_adapter(output.layout)
      out_params = self.class.output.params

      case output.layout
        when :object
          out = adapter.output(@context, ret)
          safe_ret = @authorization.filter_output(
              out_params,
              out,
              true
          )
          @reply_meta[:global].update(out.meta)

        when :object_list
          safe_ret = []

          ret.each do |obj|
            out = adapter.output(@context, obj)

            safe_ret << @authorization.filter_output(
                out_params,
                out,
                true
            )
            safe_ret.last.update({Metadata.namespace => out.meta}) unless meta[:no]
          end

        when :hash
          safe_ret = @authorization.filter_output(
              out_params,
              adapter.output(@context, ret),
              true
          )

        when :hash_list
          safe_ret = ret
          safe_ret.map! do |hash|
            @authorization.filter_output(
                out_params,
                adapter.output(@context, hash),
                true
            )
          end

        else
          safe_ret = ret
      end

      if self.class.blocking
        @reply_meta[:global][:action_state_id] = state_id
      end

      ns = {output.namespace => safe_ret}
      ns[Metadata.namespace] = @reply_meta[:global] unless meta[:no]

      [true, ns]

    else
      [true, {}]
    end

  else
    [false, @message, @errors, @http_status]
  end
end

#set_meta(hash) ⇒ Object



312
313
314
# File 'lib/haveapi/action.rb', line 312

def set_meta(hash)
  @reply_meta[:global].update(hash)
end

#v?(v) ⇒ Boolean

Returns:



365
366
367
# File 'lib/haveapi/action.rb', line 365

def v?(v)
  @version == v
end

#validate! ⇒ Object



279
280
281
282
283
284
285
# File 'lib/haveapi/action.rb', line 279

def validate!
  begin
    @params = validate
  rescue ValidationError => e
    error(e.message, e.to_hash)
  end
end