Class: HaveAPI::Action

Inherits:
Common
  • Object
show all
Includes:
Hookable
Defined in:
lib/haveapi/action.rb

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Hookable

included

Methods inherited from Common

has_attr, inherit_attrs

Constructor Details

#initialize(request, version, params, body, context) ⇒ Action

Returns a new instance of Action.



203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
# File 'lib/haveapi/action.rb', line 203

def initialize(request, version, params, body, context)
  @request = request
  @version = version
  @params = params
  @params.update(body) if body
  @context = context
  @context.action = self.class
  @context.action_instance = self
  @metadata = {}
  @reply_meta = {object: {}, global: {}}
  @flags = {}

  class_auth = self.class.authorization

  if class_auth
    @authorization = class_auth.clone
  else
    @authorization = Authorization.new {}
  end
end

Class Attribute Details

.authorization ⇒ Object (readonly)

Returns the value of attribute authorization.



20
21
22
# File 'lib/haveapi/action.rb', line 20

def authorization
  @authorization
end

.examples ⇒ Object (readonly)

Returns the value of attribute examples.



20
21
22
# File 'lib/haveapi/action.rb', line 20

def examples
  @examples
end

.resource ⇒ Object (readonly)

Returns the value of attribute resource.



20
21
22
# File 'lib/haveapi/action.rb', line 20

def resource
  @resource
end

Instance Attribute Details

#errors ⇒ Object (readonly)

Returns the value of attribute errors.



16
17
18
# File 'lib/haveapi/action.rb', line 16

def errors
  @errors
end

#flags ⇒ Object

Returns the value of attribute flags.



17
18
19
# File 'lib/haveapi/action.rb', line 17

def flags
  @flags
end

#message ⇒ Object (readonly)

Returns the value of attribute message.



16
17
18
# File 'lib/haveapi/action.rb', line 16

def message
  @message
end

#version ⇒ Object (readonly)

Returns the value of attribute version.



16
17
18
# File 'lib/haveapi/action.rb', line 16

def version
  @version
end

Class Method Details

.authorize(&block) ⇒ Object



124
125
126
# File 'lib/haveapi/action.rb', line 124

def authorize(&block)
  @authorization = Authorization.new(&block)
end

.build_route(prefix) ⇒ Object



135
136
137
138
139
140
141
142
143
# File 'lib/haveapi/action.rb', line 135

def build_route(prefix)
  route = @route || to_s.demodulize.underscore

  if !route.is_a?(String) && route.respond_to?(:call)
    route = route.call(self.resource)
  end

  prefix + route % {resource: self.resource.to_s.demodulize.underscore}
end

.delayed_inherited(subclass) ⇒ Object



33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
# File 'lib/haveapi/action.rb', line 33

def delayed_inherited(subclass)
  resource = Kernel.const_get(subclass.to_s.deconstantize)

  inherit_attrs(subclass)
  inherit_attrs_from_resource(subclass, resource, [:auth])

  i = @input.clone
  i.action = subclass

  o = @output.clone
  o.action = subclass

  m = {}

  @meta.each do |k,v|
    m[k] = v && v.clone
    next unless v
    m[k].action = subclass
  end

  subclass.instance_variable_set(:@input, i)
  subclass.instance_variable_set(:@output, o)
  subclass.instance_variable_set(:@meta, m)

  begin
    subclass.instance_variable_set(:@resource, resource)
    subclass.instance_variable_set(:@model, resource.model)
  rescue NoMethodError
    return
  end
end

.describe(context) ⇒ Object



145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
# File 'lib/haveapi/action.rb', line 145

def describe(context)
  authorization = (@authorization && @authorization.clone) || Authorization.new

  return false if (context.endpoint || context.current_user) && !authorization.authorized?(context.current_user)

  route_method = context.action.http_method.to_s.upcase
  context.authorization = authorization

  if context.endpoint
    context.action_instance = context.action.from_context(context)

    ret = catch(:return) do
      context.action_prepare = context.action_instance.prepare
    end

    return false if ret == false
  end

  {
      auth: @auth,
      description: @desc,
      aliases: @aliases,
      input: @input ? @input.describe(context) : {parameters: {}},
      output: @output ? @output.describe(context) : {parameters: {}},
      meta: @meta ? @meta.merge(@meta) { |_, v| v && v.describe(context) } : nil,
      examples: @examples ? @examples.map { |e| e.describe } : [],
      url: context.resolved_url,
      method: route_method,
      help: "#{context.url}?method=#{route_method}"
  }
end

.example(title = '', &block) ⇒ Object



128
129
130
131
132
133
# File 'lib/haveapi/action.rb', line 128

def example(title = '', &block)
  @examples ||= []
  e = Example.new(title)
  e.instance_eval(&block)
  @examples << e
end

.from_context(c) ⇒ Object



191
192
193
194
195
196
197
198
199
200
# File 'lib/haveapi/action.rb', line 191

def from_context(c)
  ret = new(nil, c.version, c.params, nil, c)
  ret.instance_exec do
    @safe_params = @params.dup
    @authorization = c.authorization
    @current_user = c.current_user
  end

  ret
end

.inherit_attrs_from_resource(action, r, attrs) ⇒ Object

Inherit attributes from resource action is defined in.



178
179
180
181
182
183
184
185
186
187
188
189
# File 'lib/haveapi/action.rb', line 178

def inherit_attrs_from_resource(action, r, attrs)
  begin
    return unless r.obj_type == :resource

  rescue NoMethodError
    return
  end

  attrs.each do |attr|
    action.method(attr).call(r.method(attr).call)
  end
end

.inherited(subclass) ⇒ Object



22
23
24
25
26
27
28
29
30
31
# File 'lib/haveapi/action.rb', line 22

def inherited(subclass)
  # puts "Action.inherited called #{subclass} from #{to_s}"

  subclass.instance_variable_set(:@obj_type, obj_type)

  if subclass.name
    # not an anonymouse class
    delayed_inherited(subclass)
  end
end

.initialize ⇒ Object



65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
# File 'lib/haveapi/action.rb', line 65

def initialize
  return if @initialized

  input.exec
  model_adapter(input.layout).load_validators(model, input) if model

  output.exec

  model_adapter(input.layout).used_by(:input, self)
  model_adapter(output.layout).used_by(:output, self)

  if @meta
    @meta.each_value do |m|
      next unless m
      m.input && m.input.exec
      m.output && m.output.exec
    end
  end

  @initialized = true
end

.input(layout = nil, namespace: nil, &block) ⇒ Object



91
92
93
94
95
96
97
98
99
100
# File 'lib/haveapi/action.rb', line 91

def input(layout = nil, namespace: nil, &block)
  if block
    @input ||= Params.new(:input, self)
    @input.layout = layout
    @input.namespace = namespace
    @input.add_block(block)
  else
    @input
  end
end

.meta(type = :object, &block) ⇒ Object



113
114
115
116
117
118
119
120
121
122
# File 'lib/haveapi/action.rb', line 113

def meta(type = :object, &block)
  if block
    @meta ||= {object: nil, global: nil}
    @meta[type] ||= Metadata::ActionMetadata.new
    @meta[type].action = self
    @meta[type].instance_exec(&block)
  else
    @meta[type]
  end
end

.model_adapter(layout) ⇒ Object



87
88
89
# File 'lib/haveapi/action.rb', line 87

def model_adapter(layout)
  ModelAdapter.for(layout, resource.model)
end

.output(layout = nil, namespace: nil, &block) ⇒ Object



102
103
104
105
106
107
108
109
110
111
# File 'lib/haveapi/action.rb', line 102

def output(layout = nil, namespace: nil, &block)
  if block
    @output ||= Params.new(:output, self)
    @output.layout = layout
    @output.namespace = namespace
    @output.add_block(block)
  else
    @output
  end
end

Instance Method Details

#authorized?(user) ⇒ Boolean

Returns:



232
233
234
235
# File 'lib/haveapi/action.rb', line 232

def authorized?(user)
  @current_user = user
  @authorization.authorized?(user)
end

#current_user ⇒ Object



237
238
239
# File 'lib/haveapi/action.rb', line 237

def current_user
  @current_user
end

#exec ⇒ Object

This method must be reimplemented in every action. It must not be invoked directly, only via safe_exec, which restricts output.



278
279
280
# File 'lib/haveapi/action.rb', line 278

def exec
  ['not implemented']
end

#input ⇒ Object



245
246
247
# File 'lib/haveapi/action.rb', line 245

def input
  @safe_params[ self.class.input.namespace ] if self.class.input
end

#meta ⇒ Object



253
254
255
# File 'lib/haveapi/action.rb', line 253

def meta
  @metadata
end

#params ⇒ Object



241
242
243
# File 'lib/haveapi/action.rb', line 241

def params
  @safe_params
end

#pre_exec ⇒ Object



272
273
274
# File 'lib/haveapi/action.rb', line 272

def pre_exec

end

#prepare ⇒ Object

FIXME: is this correct behaviour? ++



268
269
270
# File 'lib/haveapi/action.rb', line 268

def prepare

end

#request ⇒ Object



249
250
251
# File 'lib/haveapi/action.rb', line 249

def request
  @request
end

#safe_exec ⇒ Object

Calls exec while catching all exceptions and restricting output only to what user can see. Return array [status, data|error, errors]



285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
# File 'lib/haveapi/action.rb', line 285

def safe_exec
  ret = catch(:return) do
    begin
      validate!
      prepare
      pre_exec
      exec
    rescue Exception => e
      tmp = call_class_hooks_as_for(Action, :exec_exception, args: [self, e])

      if tmp.empty?
        p e.message
        puts e.backtrace
        error('Server error occurred')
      end

      error(tmp[:message]) unless tmp[:status]
    end
  end

  safe_output(ret)
end

#safe_output(ret) ⇒ Object



312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
# File 'lib/haveapi/action.rb', line 312

def safe_output(ret)
  if ret
    output = self.class.output

    if output
      safe_ret = nil
      adapter = self.class.model_adapter(output.layout)
      out_params = self.class.output.params

      case output.layout
        when :object
          out = adapter.output(@context, ret)
          safe_ret = @authorization.filter_output(
              out_params,
              out,
              true
          )
          @reply_meta[:global].update(out.meta)

        when :object_list
          safe_ret = []

          ret.each do |obj|
            out = adapter.output(@context, obj)

            safe_ret << @authorization.filter_output(
                out_params,
                out,
                true
            )
            safe_ret.last.update({Metadata.namespace => out.meta}) unless meta[:no]
          end

        when :hash
          safe_ret = @authorization.filter_output(
              out_params,
              adapter.output(@context, ret),
              true
          )

        when :hash_list
          safe_ret = ret
          safe_ret.map! do |hash|
            @authorization.filter_output(
                out_params,
                adapter.output(@context, hash),
                true
            )
          end

        else
          safe_ret = ret
      end

      ns = {output.namespace => safe_ret}
      ns[Metadata.namespace] = @reply_meta[:global] unless meta[:no]

      [true, ns]

    else
      [true, {}]
    end

  else
    [false, @message, @errors]
  end
end

#set_meta(hash) ⇒ Object



257
258
259
# File 'lib/haveapi/action.rb', line 257

def set_meta(hash)
  @reply_meta[:global].update(hash)
end

#v?(v) ⇒ Boolean

Returns:



308
309
310
# File 'lib/haveapi/action.rb', line 308

def v?(v)
  @version == v
end

#validate! ⇒ Object



224
225
226
227
228
229
230
# File 'lib/haveapi/action.rb', line 224

def validate!
  begin
    @params = validate
  rescue ValidationError => e
    error(e.message, e.to_hash)
  end
end